var def = require('./imports').default;
var THREE = def.THREE;
var Edge = def.Edge;
var Node = def.Node;
var OrbitControls = def.OrbitControls;
var d3 = def.d3;
var ee = def.ee;
var $ = require('jQuery');

module.exports = function (self) {
  self.addNode = function (node) {
    self.graph.addNode(node.id, node.data);
    var graphNode = self.graph.getNode(node.id);
    graphNode.x = node.x;
    graphNode.y = node.y;
    graphNode.fx = node.fx;
    graphNode.fy = node.fy;
    graphNode.pinnedx = node.fx != undefined;
    graphNode.pinnedy = node.fy != undefined;
    self.prepNode(graphNode);
    self.drawNode(graphNode);
    graphNode.links = [];
    self.layoutInit = true;
    return graphNode;
  };

  self.removeNode = function (node) {
    if (!node || !node.id) {
      return
    }
    self.removeGraphEdges(node)
    self.undrawEdges(node);
    self.nodes.remove(node.renderData.draw_object);
    self.graph.removeNode(node.id);
    self.undrawNode(node);
    self.nodeCount -= 1;
  }

  self.removeGraphEdges = function(node) {
    var nodeLinks = self.graph.getLinks(node.id);
    if (nodeLinks) {
      nodeLinks.forEach(function(edge) {
        self.graph.removeLink(edge)
      });
    }
  }
  
  self.undrawNode = function(node) {
    var nodes = self.force.nodes();
    if (nodes) {
      for(var i = 0; i < nodes.length; i++) {
        if(nodes[i].id == node.id) {
          self.scene.remove(nodes[i].renderData.draw_object);
          nodes[i].renderData.textHolder.children[0].element.hidden = true;
          nodes.splice(i, 1);
        }
      }
    }
  }
  
  self.undrawEdges = function(node) {
    var positions = self.edges.attributes.position.array;
    var colors = self.edges.attributes.color.array;
    var z = 0;
    for (var i = 0; i < self.lineIndices.length + z; i++) {
      if (self.lineIndices[i - z].source.id == node.id
        || self.lineIndices[i - z].target.id == node.id) {
        for (var j = i - z; j < self.lineIndices.length; j++) {
          self.lineIndices[j] = self.lineIndices[j + 1];
          self.lineObjects[j * 2] = self.lineObjects[(j + 1) * 2];
          self.lineObjects[j * 2 + 1] = self.lineObjects[(j + 1) * 2 + 1];
          var k = j + 1
          if (k == self.lineIndices.length) {
            positions[j * 3] = 0;
            positions[j * 3 + 1] = 0;
            positions[j * 3 + 3] = 0;
            positions[j * 3 + 4] = 0;
            colors[j * 3] = 255;
            colors[j * 3 + 1] = 255;
            colors[j * 3 + 2] = 255;
            colors[j * 3 + 3] = 255;
            colors[j * 3 + 4] = 255;
            colors[j * 3 + 5] = 255;
          } else {
            positions[j * 3] = positions[k * 3];
            positions[j * 3 + 1] = positions[k * 3 + 1];
            positions[j * 3 + 3] = positions[k * 3 + 3];
            positions[j * 3 + 4] = positions[k * 3 + 4];
            colors[j * 3] = colors[k * 3];
            colors[j * 3 + 1] = colors[k * 3 + 1];
            colors[j * 3 + 2] = colors[k * 3 + 2];
            colors[j * 3 + 3] = colors[k * 3 + 3];
            colors[j * 3 + 4] = colors[k * 3 + 4];
            colors[j * 3 + 5] = colors[k * 3 + 5];
          }
        }
        self.lineIndices.pop();
        self.lineObjects.pop();
        self.lineObjects.pop();
        self.drawCount -= 2;
      }
    }
  }

  self.getNode = function (node_id) {
    return self.graph.getNode(node_id);
  };
  
  self.addEdge = function (source, target, visible=true) {
    self.graph.addLink(source.id, target.id);
    self.drawEdge(self.getNode(source.id), self.getNode(target.id), visible);
  };

  self.addNeighbors2 = function(node, neighbors) {
    for (var i = 0; i < neighbors.length; i++) {
      var newNode = neighbors[i];
      self.addNode(newNode);
      self.addEdge(node.id, newNode.id);
    }
  }

  self.addNeighbors = function(node) {
    self.removeFakes();
    self.neighborHost = node;
    node.pinnedx = true;
    node.pinnedy = true;
    var id = node.id;
    var points = [];
    for (var i = 0; i < 5; i++) {
      id += 10000;
      var newNode = $.extend(true, {}, node);
      newNode.id = id;
      newNode.data.label = i;
      newNode.fx = node.x;
      newNode.fy = node.y;
      newNode.data.shape = node.renderData.shape;
      self.fakeNodes.push(self.addNode(newNode));
      self.addEdge(self.graph.getNode(node.id), self.graph.getNode(newNode.id));
    }
    self.lastNode = self.graph.getNode(id);
  }

  self.removeFakes = function(node=null) {
    if (self.fakeNodes.length >= 3) {
      for (var i = 0; i < 5; i++) {
        if (i != node) {
          self.removeNode(self.fakeNodes[i]);
        }
        self.fakeNodes[i].pinnedx = false;
        self.fakeNodes[i].pinnedy = false;
      }
      self.fakeNodes = []
    }
    if (self.neighborHost) {
      self.neighborHost.pinnedx = false;
      self.neighborHost.pinnedy = false;
      self.neighborHost = null;
    }
  }
}